#ifndef LOW_LEVEL_PLATFORM_H_
#define LOW_LEVEL_PLATFORM_H_

#include <Core/Base/NoInstanceBase.h>
#include <SimulationEnvironment.h>
#include <OS/MainLoop/DefaultSimulationPowerSaveCallback.h>

/**
 * This class provides a low layer hardware abstraction, making the
 * layers above (high level drivers and application) independent of the
 * platform they are running on.
 * It defines the low level drivers for the simulation environment.
 */
template< unsigned int T = 0 >
class TLowLevelPlatform  : public redBlocks::Core::NoInstanceBase
{
  public:

    // @todo: insert typedefs for all platform devices

    // define communication interface 0 for logging output
    typedef redBlocks::HAL::Drivers::TComInterfaceDriverStub<0> TerminalComInterface;

    // provides the static methods enableCallback() and disableCallback() in order to switch
    // periodic system tick callback on and off
    typedef redBlocks::Simulators::SimulationStubs::SystemTick SystemTickDriver;
    // provides the static methods lock() and unlock() that are used to lock all interrupts
    typedef redBlocks::Simulators::SimulationStubs::IsrLock IsrLock;
    // provides the means to the mainloop scheduler to control the processor's sleep mode
    typedef redBlocks::OS::MainLoopScheduler::DefaultSimulationPowerSaveCallback PowerSaveCallback;

    typedef redBlocks::HAL::Drivers::TDigitalOutputDriverStub<0> Led;
    typedef redBlocks::HAL::Drivers::TDigitalOutputDriverStub<1> AwakeLed;

    /**
     * Entering the sleep mode is delegated to the simulation client
     */
    static void enterSleepMode()
    {
      redBlocks::Simulators::SimulationStubs::SleepMode::enter();
    }

    static void initLowLevelDrivers()
    {
      typedef redBlocks::Simulators::SimulationStubs SimulationStubs;

      // configure system ticks: per simulation tick we generate 10 system ticks (which results in
      // a simulated system tick period of 1 ms, if 100 simulation ticks are configured in main.cpp)
      SimulationStubs::SystemTick::configure( 10, 1 );

      // configure communication interface that is used for logging (hardware receive and send queue is
      // simulated to be 1 byte long) on a real target TerminalComInterface would most likely be a UART interface
      SimulationStubs::ComInterface::add <TerminalComInterface> ( 1, 1 );

      //@todo: insert driver stub initialization code
      SimulationStubs::DigitalOutput::add<Led>( false );
      SimulationStubs::DigitalOutput::add<AwakeLed>( false );
    }
};

typedef TLowLevelPlatform<> LowLevelPlatform;

#endif
