#include <Platform.h>
#include <PlatformCallbacks.h>
#include <Application.h>
#include <Core/Log/Log.h>
#include <Memory/HeapManager/HeapManager.h>

#include <Simulators/ClientPosixTcpSimulatorConnection.h>
#include <Simulators/TPosixSimulationClientEnvironmentThread.h>
#include <OS/Posix/Posix.h>

/**
 * Main routine of our simulator application. It sets up the single components,
 * associates them with each other and starts the application.
 */
int main()
{
  typedef redBlocks::Memory::HeapManager::THeapManagerStaticAlloc<12000> HeapManager;
  HeapManager::init();

  // Connect to the simulator via TCP connection.
  redBlocks::Simulators::ClientPosixTcpSimulatorConnection simulatorConnection;
  simulatorConnection.connect( "127.0.0.1", 10000 );

  // Setup the simulation client environment.
  // We choose the thread based version, as we are working on top of an operating system.
  redBlocks::Simulators::TPosixSimulationClientEnvironmentThread<> simClientEnv;

  // Initialize the simulation client.
  // The system tick needs to be enabled before entering this method, otherwise this method will never return.
  redBlocks::Simulators::SimulationClient simulationClient( simulatorConnection, simClientEnv );

  // Driver initialization.
  Platform::initLowLevelDrivers();
  Platform::initHighLevelDrivers();

  // Initialization of all simulation stubs finished.
  simulationClient.initDone();

  // Initialize application.
  Application app;

  // Prevent dynamic memory allocation after this point.
  HeapManager::disableAllocation();

  RB_LOG_DEBUG( "Heap memory used: " << HeapManager::getUsed() );
  RB_LOG_DEBUG( "Application initialized, starting event processing" );

  app.run();

  return 0;
}


