#include <string.h>

#include "XMC1100.h"
#include <Platform.h>
#include <Application.h>
#include <PlatformCallbacks.h>
#include <Core/Log/Log.h>
#include <Memory/HeapManager/HeapManager.h>

extern "C"
{
  void USIC0_0_IRQHandler()
  {
    Platform::TerminalComInterface::onSendIsr();
  }

  void USIC0_1_IRQHandler()
  {
    Platform::TerminalComInterface::onRecvIsr();
  }

  void SysTick_Handler( void )
  {
    Platform::SystemTickDriver::onIsr();
  }
}

int main( void )
{
  // Clock configuration
  SCU_GENERAL->PASSWD = 0x000000C0UL;
  SCU_CLK->CLKCR = 0x3FF00400UL; // 8 MHz MCLK, 8 MHz PCLK
  SCU_GENERAL->PASSWD = 0x000000C3UL;
  SystemCoreClockUpdate();

  // We configure the heap size to 2000 bytes
  typedef redBlocks::Memory::HeapManager::THeapManagerStaticAlloc<2000> HeapManager;
  HeapManager::init();

  // Driver initialization.
  Platform::initLowLevelDrivers();
  Platform::initHighLevelDrivers();

  // Initialize application.
  Application app;

  // Prevent dynamic memory allocation after this point.
  HeapManager::disableAllocation();
  RB_LOG_DEBUG( "Heap memory used: " << HeapManager::getUsed() );
  RB_LOG_DEBUG( "Application initialized, starting event processing" );

  app.run();
}
