#ifndef PLATFORM_H_
#define PLATFORM_H_

#include <LowLevelPlatform.h>
#include <Core/Log/TStandardLogger.h>
#include <HAL/TBufferedComInterfaceWriter.h>
#include <OS/MainLoop/TMainLoopScheduler.h>

/**
 * This class enhances the low layer hardware abstraction with high level (and thus
 * hardware independent) driver functionality.
 */
class Platform : public LowLevelPlatform
{
  public:
    /**
     * TMainLoopScheduler, instantiated with the right template arguments is used as our operating system
     */
    typedef redBlocks::OS::TMainLoopScheduler<LowLevelPlatform::IsrLock, 1, 1, LowLevelPlatform::PowerSaveCallback> OS;

    /**
     * high level driver that provides data buffering on top of the communication interface that is
     * used for logging
     */
    typedef redBlocks::HAL::TBufferedComInterfaceWriter<Platform::TerminalComInterface> BufferedLogComInterfaceWriter;

    /**
     * the logger that is used as the standard logger
     */
    typedef redBlocks::Core::Log::TGlobalStandardLogger<Platform::BufferedLogComInterfaceWriter> StandardLogger;

    static void initHighLevelDrivers()
    {
      // initializes the high level driver that buffers logging output with a buffer size of 400 bytes
      new BufferedLogComInterfaceWriter( 400 );
      // initializes the standard logger
      new StandardLogger();
    }
};

// callback routing from low level drivers to high level drivers
// @todo: insert routing with RB_CONNECT_ISR_CBK as required

// associate the method tick() that is provided by TMainLoopScheduler with the SystemTickDriver
RB_CONNECT_ISR_CBK( Platform::SystemTickDriver, Platform::SystemTickDriver::CBK_ON_SYSTEM_TICK, Platform::OS::tick() )
// associate the send callback of the communication interface that is used for logging output with the high level driver (the
// receive callback mapping is defined in PlatformCallbacks.h)
RB_CONNECT_ISR_CBK( Platform::TerminalComInterface, Platform::TerminalComInterface::CBK_ON_SEND_READY, Platform::BufferedLogComInterfaceWriter::onSendReadyCallback() )

#endif // PLATFORM_H_
