#ifndef GLOBAL_FIXTURE_H_
#define GLOBAL_FIXTURE_H_

#include <cxxtest/TestSuite.h>
#include <cxxtest/GlobalFixture.h>

/**
 * This class allows to specify custom setup and teardown methods, for CxxTest.
 * The setUp method for all global fixtures is called before each test is executed,
 * and the tearDown method for all global fixtures is called after each test is completed.
 *
 * This file also specify the standard logger, which is used to output log messages during unit test.
 */

class Fixture : public CxxTest::GlobalFixture
{
  public:

    /** Called before each test is executed */
    bool setUp()
    {
      return true;
    }

    /** Called after each test is executed */
    bool tearDown()
    {
      return true;
    }


    /** Called before testsuite is executed */
    bool setUpWorld()
    {
      printf( "-------------------\n" );
      return true;
    }


    /** Called after testsuite is executed */
    bool tearDownWorld()
    {
      printf( "-------------------\n" );
      printf( "Finished test suite\n" );
      return true;
    }
};
static Fixture fixture;

#endif // GLOBAL_FIXTURE_H_
